program CardView

  use ReadCB_m, only: OpenCB, ReadRaw
  use Winteracter

  implicit None

  ! Dot pitch
  real, parameter :: DPI = 94.11 ! Dots per inch

  ! Window dimensions
  real, parameter :: CardWidth = 7.375, CardHeight = 3.25   ! Inches

  ! Colors
  integer, parameter :: BGColor = 245 + 256*222 + 256*256*179 ! Wheat
  integer, parameter :: FGcolor = 0                           ! Black

  ! Fill pattern for "punches"
  integer, parameter :: Dense = 4

  ! Menu stuff
  integer, parameter :: IDM_Menu1  = 30001 ! The menu
  integer, parameter :: IDM_File   = 40001 ! File name
  integer, parameter :: IDM_CardNo = 40002 ! Card number
  integer, parameter :: IDM_Exit   = 40003 ! Exit

  ! Dialogue stuff
  integer, parameter :: IDD_Dialog = 101  ! Dialog box
  integer, parameter :: IDF_String = 1001 ! String box within dialog box
  integer, parameter :: MyOK       = 1002 ! Dialog box OK button
  integer, parameter :: MyCancel   = 1003 ! Dialog box Cancel button

  integer :: Button             ! From the dialog box
  integer :: COLS(0:79) = 0     ! Columns 1:80 from the card
  character(len=6) :: CardNoTxt ! From the dialog
  integer :: CardNo = 1         ! From -r or the dialog via READ statement
  character(len=1023) :: Fname  ! File name
  integer :: I                  ! Subscript for command-line arguments
  integer :: ITYPE              ! Window event type
  character(len=1023) :: Line
  type(win_style) :: WINDOW
  type(win_message) :: MESSAGE

! Process command line
  i = 0
  fname = ''
  do
    i = i + 1
    call getarg ( i, line )
    if ( line(1:2) == '-r' ) then
      if ( line(3:3) == '' ) then
        i = i + 1
        call getarg ( i, line(3:) )
      end if
      read ( line(3:), * ) cardNo
    else if ( line(1:1) == '-' ) then
      call getarg ( 0, line )
      print *, 'Usage: ', trim(line), ' [options] [file]'
      print *, '  Options: -r[ ]rec => Show record number rec'
      print *, '           -<anything else> => Show usage'
      stop
    else if ( line == '' ) then
  exit
    else
      fname = line
      call openCB ( fname, 10 )
    end if
  end do

  if ( fname /= '' ) call readRaw ( 10, cardNo, cols )

! Initialize WiSK
  call WInitialise ( )

! Set up root window options:

! Set up root window options:

!     -  System menu on
!     -  Minimise button
!     -  Maximise button

  window%flags  = SysMenuOn + MinButton + MaxButton + FixedSizeWin

! Centre window in the screen
  window%x      = -1
  window%y      = -1

! Window Size
  window%width  = nint(cardWidth*dpi)
  window%height = nint(cardHeight*dpi)

! Set resource menu id and window title
  window%menuid = idm_menu1
  window%title  = 'Cardview'

  call windowOpen ( window )

  call IGrUnits ( 0.0, 0.0, cardWidth, cardHeight )

  call IGrColourModel ( 24 )
  call IGrColourN ( fgColor )
  call IGrFillPattern ( dense )

  call WDialogLoad ( IDD_Dialog )
  do
    call WMessage ( itype, message )
    select case ( itype )

    case ( closeRequest )
  exit

    case ( expose )
      call drawCard

    case ( menuSelect )
      select case ( message%value1 )
      case ( IDM_CardNo )
        call WDialogSelect ( IDD_Dialog )
        call WDialogShow ( itype=modal )
      case ( IDM_Exit )
  exit
      case ( IDM_File )
        call WSelectFile ( '', PromptOn, fname, 'Load Deck' )
        if ( WInfoDialog(ExitButtonCommon) == CommonOK ) then
!         print *, 'Input file name is ', trim(fname)
          close ( 10, iostat=i ) ! Ignore status
          call openCB ( fname, 10 )
        end if
      end select

    case ( pushButton )
      if ( WInfoDialog(exitButton) == MyOK ) then
        call WDialogGetString ( IDF_String, CardNoTxt )
        read ( cardNoTxt, *, iostat=i ) cardNo
        if ( i /= 0 ) then
          call WDialogSelect ( IDD_Dialog )
          call WDialogShow ( itype=modal )
        else
          call readRaw ( 10, cardNo, cols )
          call drawCard
        end if
      end if
      call WDialogHide

    end select

  end do

  call windowClose ( )

contains

  ! ---------------------------------------------------  DrawCard  -----
  subroutine DrawCard

    ! Hole dimensions
    real, parameter :: HoleWidth = 0.055, HoleHeight = 0.125  ! Inches
    real, parameter :: BotMargin = 0.250, LeftMargin = 0.251 ! inches
    real, parameter :: HorizSpace = 0.087, VertSpace = 0.250  ! Inches

    ! Draw a hole
    real :: Xhole(4) = (/ 0.0, holeWidth, holeWidth, 0.0 /)
    real :: Yhole(4) = (/ 0.0, 0.0, holeHeight, holeHeight /)

    ! Subscripts / loop inductors
    integer :: Row             ! Rows 0:11, numbered from the bottom
    integer :: Column          ! Columns 0:79, numbered from the left

    call windowClear ( rgb = bgcolor ) ! Background
    do row = 0, 11
      do column = 0, 79
        if ( ibits(cols(column),row,1) /= 0 ) &
        &  call IGrPolygonComplex ( &
          & XHole + leftMargin - 0.5*holeWidth + horizSpace*Column, &
          & YHole + botMargin - 0.5*holeHeight + vertSpace*row, 4 )
      end do ! column
    end do ! row

  end subroutine DrawCard

  ! ---------------------------------------------------  OpenDeck  -----

end program CardView
